-- Additional runners. Orchestrate

global_runner = nil
function next_runner(run)
	if global_run then
		global_run:die()
	end
	global_run = run
	if global_run then
		global_run:insert()
	end
end

function any_contents(t)
	for k,v in pairs(t) do
		return true
	end
	return false
end

local fileversion = 1
gallery = {version=fileversion, best={}}
local gallery_path = "gallery/records.dat"
do
	local data = read_file_table(gallery_path)
	if data and data.version == fileversion then gallery = data end
end
function save_gallery()
	pretty.dump(gallery, gallery_path)
end
function gallery_image_for(key)
	return string.format("gallery/best-%d.png", key)
end

class.FreeRunner(Runner)

function FreeRunner:_init(spec)
	self:super(spec)
	
	-- Find correct scale
	if not self.scale then
		self.scale = 1
		while true do
			local scale = self.scale + 1
			local tw, th = self.w*scale, self.h*scale
			if tw >= engine.sw or th >= engine.sh then break end
			self.scale = scale
		end
	end
	
	-- Find correct blit point
	self.sx = (engine.sw - self.w*self.scale)/2
	self.sy = (engine.sh - self.h*self.scale)/2
end

-- for running two comparison images
class.CoolRunner(Runner)

function CoolRunner:_init(spec)
	self:super(spec)
	
	-- Find correct scale
	if not self.scale then
		self.scale = 1
		while true do
			local scale = self.scale + 1
			local tw, th = self.w*scale*2, self.h*scale
			if tw >= engine.sw or th >= engine.sh then break end
			self.scale = scale
		end
	end

	-- Find correct blit point
	self.sx = engine.sw/4-self.w*self.scale/2
	self.sy = (engine.sh - self.h*self.scale)/2
	self.sx2 = 3*engine.sw/4-self.w*self.scale/2
	self.tx = self.sx
	self.ty = self.sy-engine.fh*2
	self.txright = self.sx2  + self.h*self.scale
	self.titlex = self.sx
	self.titley = self.sy + self.h*self.scale + engine.fh
	self.titlexright = self.txright

	self.titley2 = self.titley + engine.fh * 2
end

function CoolRunner:insert(prio)
	Runner.insert(self,prio)
	
	self.coolpic = self.coolpic or MapLoad(self.target or "resource/stage/lesson_2.png")
	if self.tutor then
		table.insert(self.die_with, self.tutor)
		self.tutor.stage = self.stage
		self.tutor:insert()
	end
end

function CoolRunner:die()
	MapKill(self.coolpic)
	
	Runner.die(self)
end

function CoolRunner:remaining()
	local now = engine.sdl.SDL_GetTicks()
	if not self.started then self.started = now end
	local remain = self.started-now + self.limit
--	print(string.format("now %s started %s limit %s remain %s um %s um?? %d\n", now, self.started, self.limit, remain, math.floor(remain/1000), math.floor(remain/1000)))
	if remain < 0 then return nil
	else return string.format("  %d:%02d", math.floor(remain/1000), math.floor((remain%1000)/1000 * 60)) end
end

function CoolRunner:should_superfinish()
	local now = engine.sdl.SDL_GetTicks()
	if not self.started then return true end
	local remain = self.started-now + self.limit + 4000
	return remain < 0
end

function CoolRunner:should_skip()
	local now = engine.sdl.SDL_GetTicks()
	if not self.started then return true end
	local remain = self.started-now + self.limit + 60000
	return remain < 0
end

-- tx, ty: Upper left
-- titlex, titley: Bottom left

function CoolRunner:onTick()
	local function draw()
		engine.clear_screen()
		engine.blit_whole_scaled(engine.screen, self.stage.draw, self.scale, self.sx, self.sy)
		engine.blit_whole_scaled(engine.screen, self.coolpic, self.scale, self.sx2, self.sy)
		if self.title then engine.draw_string(self.title, self.tx, self.ty) end
	end

	local function draw_similarity()
		local f = string.format("%.2f", self.n)
		while #f < 6 do f = " " .. f end
		engine.draw_string(string.format("YOUR SCORE: %s%% SIMILAR", f), self.titlex, self.titley)
	end

	if not self.shouldstart then
		draw()
		engine.draw_string(string.format("BECOME A GREAT ARTIST IN JUST 10 SECONDS", n), self.tx + 21*engine.fw, self.ty - engine.fh*2)
		engine.draw_string_right("10.00", self.txright, self.ty)
		engine.draw_string(string.format("To paint: Use keyboard", f), self.titlex, self.titley)
		engine.draw_string_right(string.format("How close can you get?"), self.titlexright, self.titley)
		engine.draw_string(string.format("-- Press key to start --", n), self.titlex + 29*engine.fw, self.titley2)
		if any_contents(engine.ui_state.pressed) then
			project.sound_start(self.music)
			self.shouldstart = true
		end
	elseif not self.finished then
		draw()
		self.n = project.compute_similarity(self.stage.base, self.coolpic)
		draw_similarity()
		engine.draw_string_right(string.format("How close can you get?"), self.titlexright, self.titley)
		-- if self.best then engine.draw_string_right(string.format("Personal Best: %.2f%%", self.best.similar), self.titlexright, self.titley) end
		if self.limit then
			local remain = self:remaining()
			if remain then
				engine.draw_string_right(remain, self.txright, self.ty)
			else
				self.finished = true
				engine.draw_string_right("COMPLETE!", self.txright, self.ty)
				if not gallery.best[self.scorekey] or gallery.best[self.scorekey].similar < self.n then
					gallery.best[self.scorekey] = {similar=self.n, when=os.time()}
					engine.draw_string_right("              NEW BEST", self.titlexright, self.titley)
					save_gallery()
					project.save_image(self.stage.base, gallery_image_for(self.scorekey))
				else
					engine.draw_string_right(string.format("CURRENT HIGH SCORE: %.2f%%", self.best.similar), self.titlexright, self.titley)
				end
				if self.music then project.sound_release(self.music) self.music = nil end
				project.playsample_set(sampleplayer, ogg_named("time_up"), 1, 0)
				-- YOU WIN!!
				engine.draw_string(string.format("YOU DID GREAT!", n), self.titlex + 34*engine.fw, self.titley2)
			end
		end
		if self.title then engine.draw_string(self.title, self.tx, self.ty) end
	elseif not self.superfinished then
		if self:should_superfinish() then
			self.superfinished = true
			engine.draw_string(string.format("YOU DID GREAT!", n), self.tx + 34*engine.fw, self.ty - engine.fh*2)
			engine.draw_string(string.format("-- Any key to continue --", n), self.titlex + 29*engine.fw, self.titley2)
		end
	else
		if any_contents(engine.ui_state.pressed) or self:should_skip() then
			return_to_title()
		end
	end
end

require("project/title")